USE ChangeDataCapture
GO
DECLARE @from_lsn binary(10), 
	@to_lsn binary(10)
-- Map the time interval to a change data capture query range.
SELECT @from_lsn = sys.fn_cdc_map_time_to_lsn('smallest greater than', GETDATE()-1)
SELECT @to_lsn = sys.fn_cdc_map_time_to_lsn('largest less than or equal', GETDATE())
-- Return the changes occurring within the query window.
SELECT * FROM cdc.fn_cdc_get_all_changes_dbo_Note(@from_lsn, @to_lsn, 'all update old')
GO

-------------------------------------------------
-- Operation values
-- 1 Delete
-- 2 Insert
-- 3 Update (before)
-- 4 Update (after)
-------------------------------------------------

-- DROP TABLE NoteWarehouse

-- Create a simple data warehouse - this will take a long time for millions of rows
SELECT Author, NoteCount=COUNT(*), TotalLength=SUM(LEN(NoteText))
INTO NoteWarehouse
FROM dbo.Note
GROUP BY Author

SELECT * FROM NoteWarehouse

DECLARE @to_lsn binary(10) 
SELECT @to_lsn = sys.fn_cdc_get_max_lsn()
EXEC sp_cdc_cleanup_change_table 
	@capture_instance = 'dbo_Note',
	@low_water_mark=@to_lsn

------------------------------

CREATE PROC ProcessChageDataCapture
AS
DECLARE @from_lsn binary(10)
  ,@to_lsn binary(10) 

SELECT @from_lsn = sys.fn_cdc_get_min_lsn('dbo_Note')
-- We already processed this one, start with the next one
SELECT @from_lsn = sys.fn_cdc_increment_lsn(@from_lsn)
SELECT @to_lsn = sys.fn_cdc_get_max_lsn()

IF @from_lsn <= @to_lsn BEGIN
	SELECT Author, NoteLength=LEN(NoteText), 
		AddSubtract = CASE __$operation
		WHEN 1 THEN -1	-- Delete
		WHEN 2 THEN 1	-- Insert
		WHEN 3 THEN -1	-- Before Update (treat like Delete)
		WHEN 4 THEN 1	-- After Update (treat like Insert)
		END
	INTO #Transactions
	FROM cdc.fn_cdc_get_all_changes_dbo_Note(@from_lsn, @to_lsn, 'all update old');

	SELECT Author, NoteCount=SUM(AddSubtract), TotalLength=SUM(NoteLength * AddSubtract)
	INTO #Summary
	FROM #Transactions
	GROUP BY Author

	UPDATE NoteWarehouse
	SET NoteCount = w.NoteCount + s.NoteCount,
		TotalLength = w.TotalLength + s.TotalLength
	FROM NoteWarehouse w
	JOIN #Summary s on w.Author = s.Author

	INSERT NoteWarehouse
	SELECT Author, NoteCount, TotalLength
	FROM #Summary
	WHERE NOT EXISTS (SELECT * FROM NoteWarehouse where Author=#Summary.Author)

	DROP TABLE #transactions
	DROP TABLE #Summary

	EXEC sp_cdc_cleanup_change_table 
		@capture_instance = 'dbo_Note',
		@low_water_mark=@to_lsn
END
GO

-----

EXEC ProcessChageDataCapture
GO
SELECT * FROM NoteWarehouse
