USE ChangeTracking
GO

CREATE TABLE LastChangeProcessed (TableName sysname, LastSyncVersion int)
GO
insert LastChangeProcessed select 'dbo.Note', CHANGE_TRACKING_MIN_VALID_VERSION(OBJECT_ID('dbo.Note'))

update  LastChangeProcessed
set lastsyncversion=0

GO

---------------------------------------------------------------------------------------------

ALTER PROC ProcessChangeTracking
AS
SET TRANSACTION ISOLATION LEVEL SNAPSHOT
BEGIN TRANSACTION
DECLARE @LastSyncVersion int
DECLARE @CurrentSyncVersion int

SELECT @LastSyncVersion=LastSyncVersion
FROM LastChangeProcessed
WHERE TableName='dbo.Note'

-- Check whether the value for LastSyncVersion is valid.
IF (@LastSyncVersion < CHANGE_TRACKING_MIN_VALID_VERSION(OBJECT_ID('dbo.Note')))
  BEGIN
    RAISERROR (N'LastSyncVersion too old', 10, 1)
  END ELSE BEGIN
	-- Obtain the current version (used for the next sync).
    SET @CurrentSyncVersion = CHANGE_TRACKING_CURRENT_VERSION()
	-- Obtain incremental changes by using the previous sync version.
    SELECT 
      CT.ID, n.NoteDate, n.Author, n.NoteText,
      CT.SYS_CHANGE_VERSION,
      CT.SYS_CHANGE_OPERATION, CT.SYS_CHANGE_COLUMNS
   
    FROM dbo.Note n
    RIGHT OUTER JOIN CHANGETABLE(CHANGES dbo.Note, @LastSyncVersion) CT
      ON n.ID = CT.ID
    -- Do whatever needs to be done with the changed data
    
    -- Save the last version we processed
	UPDATE LastChangeProcessed
	SET LastSyncVersion=@CurrentSyncVersion
	WHERE TableName='dbo.Note'
END
COMMIT TRANSACTION
GO
---------------------------------------------------------------------------------------------

EXEC ProcessChangeTracking
GO
